@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2002 Red Hat, Inc.
+ * Copyright (C) 2002,2003 Red Hat, Inc.
  *
  * This is free software; you can redistribute it and/or modify it under
  * the terms of the GNU Library General Public License as published by
@@ -16,7 +16,7 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-#ident "$Id: iso2022.c,v 1.18 2003/01/17 17:17:56 nalin Exp $"
+#ident "$Id: iso2022.c,v 1.20.2.2 2003/04/21 20:25:57 nalin Exp $"
 #include "../config.h"
 #include <sys/types.h>
 #include <stdio.h>
@@ -30,7 +30,7 @@
 #include "iso2022.h"
 
 /* Maps which jive with XTerm's ESC ()*+ ? sequences and RFC 1468. */
-#define NARROW_MAPS	"0AB4C5RQKYE6ZH7=" "J"
+#define NARROW_MAPS	"012AB4C5RQKYE6ZH7=" "J"
 /* Maps which jive with RFC 1468's ESC $ ? sequences. */
 #define WIDE_MAPS	"@B"
 /* Maps which jive with RFC 1557/1922/2237's ESC $ ()*+ ? sequences. */
@@ -232,9 +232,9 @@ static const struct _vte_iso2022_map _vte_iso2022_map_wide_A[] = {
 static const struct _vte_iso2022_map _vte_iso2022_map_wide_B[] = {
 #include "unitable.JIS0208"
 };
-/* Korean.  KSC 5601, per RFC 1557. */
+/* Korean.  KS X 1001, per RFC 1557. */
 static const struct _vte_iso2022_map _vte_iso2022_map_wide_C[] = {
-#include "unitable.KSC5601"
+#include "unitable.KSX1001"
 };
 /* Japanese.  JIS X 0212-1990, per RFC 2237. */
 static const struct _vte_iso2022_map _vte_iso2022_map_wide_D[] = {
@@ -338,8 +338,8 @@ _vte_iso2022_map_get(gunichar mapname)
 			ret = _vte_iso2022_map_init(_vte_iso2022_map_A,
 						    G_N_ELEMENTS(_vte_iso2022_map_A));
 			break;
-		case '1':
-		case '2':
+		case '1': /* treated as an alias in xterm */
+		case '2': /* treated as an alias in xterm */
 		case 'B':
 			ret = _vte_iso2022_map_init(_vte_iso2022_map_B,
 						    G_N_ELEMENTS(_vte_iso2022_map_B));
@@ -474,6 +474,37 @@ _vte_iso2022_substitute_single(gunichar mapname, gunichar c)
 	if (mapname == '0') {
 		if ((result >= 0x2500) && (result <= 0x257f)) {
 			width = 1;
+		} else
+		switch (c) {
+		case 0x00a3: /* british pound */
+		case 0x00b0: /* degree */
+		case 0x00b1: /* plus/minus */
+		case 0x00b7: /* bullet */
+		case 0x03c0: /* pi */
+		case 0x2190: /* left arrow */
+		case 0x2191: /* up arrow */
+		case 0x2192: /* right arrow */
+		case 0x2193: /* down arrow */
+		case 0x2260: /* != */
+		case 0x2264: /* <= */
+		case 0x2265: /* >= */
+		case 0x23ba: /* scanline 1/9 */
+		case 0x23bb: /* scanline 3/9 */
+		case 0x23bc: /* scanline 7/9 */
+		case 0x23bd: /* scanline 9/9 */
+		case 0x2409: /* HT symbol */
+		case 0x240a: /* LF symbol */
+		case 0x240b: /* VT symbol */
+		case 0x240c: /* FF symbol */
+		case 0x240d: /* CR symbol */
+		case 0x2424: /* NL symbol */
+		case 0x2592: /* checkerboard */
+		case 0x25ae: /* solid rectangle */
+		case 0x25c6: /* diamond */
+			width = 1;
+			break;
+		default:
+			break;
 		}
 	}
 	/* Save the width if one was set. */
@@ -881,6 +912,37 @@ _vte_iso2022_substitute(struct _vte_iso2022 *outside_state,
 			if (current_map == '0') {
 				if ((result >= 0x2500) && (result <= 0x257f)) {
 					width = 1;
+				} else
+				switch (result) {
+				case 0x00a3: /* british pound */
+				case 0x00b0: /* degree */
+				case 0x00b1: /* plus/minus */
+				case 0x00b7: /* bullet */
+				case 0x03c0: /* pi */
+				case 0x2190: /* left arrow */
+				case 0x2191: /* up arrow */
+				case 0x2192: /* right arrow */
+				case 0x2193: /* down arrow */
+				case 0x2260: /* != */
+				case 0x2264: /* <= */
+				case 0x2265: /* >= */
+				case 0x23ba: /* scanline 1/9 */
+				case 0x23bb: /* scanline 3/9 */
+				case 0x23bc: /* scanline 7/9 */
+				case 0x23bd: /* scanline 9/9 */
+				case 0x2409: /* HT symbol */
+				case 0x240a: /* LF symbol */
+				case 0x240b: /* VT symbol */
+				case 0x240c: /* FF symbol */
+				case 0x240d: /* CR symbol */
+				case 0x2424: /* NL symbol */
+				case 0x2592: /* checkerboard */
+				case 0x25ae: /* solid rectangle */
+				case 0x25c6: /* diamond */
+					width = 1;
+					break;
+				default:
+					break;
 				}
 			}
 			/* Save the width if one was set. */
